/**
 * A named type.
 */
export interface ABIVariable {
    /**
     * The name of the variable.
     */
    name: string;
    /**
     * The type of the variable.
     */
    type: ABIType;
}
/**
 * Indicates whether a parameter is public or secret/private.
 */
export declare enum ABIParameterVisibility {
    PUBLIC = "public",
    SECRET = "secret"
}
/**
 * A function parameter.
 */
export interface ABIParameter extends ABIVariable {
    /**
     * Indicates whether a parameter is public or secret/private.
     */
    visibility: ABIParameterVisibility;
}
/**
 * A basic type.
 */
export interface BasicType<T extends string> {
    /**
     * The kind of the type.
     */
    kind: T;
}
/**
 * A variable type.
 */
export type ABIType = BasicType<'field'> | BasicType<'boolean'> | IntegerType | ArrayType | StringType | StructType;
/**
 * An integer type.
 */
export interface IntegerType extends BasicType<'integer'> {
    /**
     * The sign of the integer.
     */
    sign: string;
    /**
     * The width of the integer in bits.
     */
    width: number;
}
/**
 * An array type.
 */
export interface ArrayType extends BasicType<'array'> {
    /**
     * The length of the array.
     */
    length: number;
    /**
     * The type of the array elements.
     */
    type: ABIType;
}
/**
 * A string type.
 */
export interface StringType extends BasicType<'string'> {
    /**
     * The length of the string.
     */
    length: number;
}
/**
 * A struct type.
 */
export interface StructType extends BasicType<'struct'> {
    /**
     * The fields of the struct.
     */
    fields: ABIVariable[];
}
/**
 * Noir function types.
 */
export declare enum FunctionType {
    SECRET = "secret",
    OPEN = "open",
    UNCONSTRAINED = "unconstrained"
}
/**
 * The ABI entry of a function.
 */
export interface FunctionAbi {
    /**
     * The name of the function.
     */
    name: string;
    /**
     * Whether the function is secret.
     */
    functionType: FunctionType;
    /**
     * Whether the function is internal.
     */
    isInternal: boolean;
    /**
     * Function parameters.
     */
    parameters: ABIParameter[];
    /**
     * The types of the return values.
     */
    returnTypes: ABIType[];
    /**
     * The ACIR bytecode of the function.
     */
    bytecode: string;
    /**
     * The verification key of the function.
     */
    verificationKey?: string;
}
/**
 * Defines ABI of a contract.
 */
export interface ContractAbi {
    /**
     * The name of the contract.
     */
    name: string;
    /**
     * The functions of the contract.
     */
    functions: FunctionAbi[];
}
//# sourceMappingURL=abi.d.ts.map